CON{{Header110430a.spin 
  Modified G101011c.spin

  This file contains the costants used in many objects.
  The objects on used with the following boards.
  
  Boards
  A Main data collection board.  Records information to uSD card and transmits the
    information to board H.  
  B This gathers data from serial lines and transfer to board A.
  C VGA display board.  Also has an Emic
  D Controls LED Array and seven segment display.  Receives data from C (or should it be A?).
  E Wireless barcode reader.
  F Input devices; encoder knobs and buttons, keyboard, Nordic tranceiver.  Slave to board G.
  G This board.  Propeller Touchscreen Platform used to control, monitor (and record?) lab and household equipment.
  H Basestation.  
  *I Drying oven controller.
  *J Robot arm controller.
  U Mecamun remote
  V Mecanum vision board
  W Mecanum motor board
  *X Roboni I controller?
  *Y Kitchen over controller.
  Z Sprinkler controller.  (Maybe use other naming system for household boards.)
  


  '' Communication protocol.  Board to board.
  '' Hard wired boards:
  '' _StartOfHeader _StartOfText (destination board letter*1) (origination board letter)(_Enquire, _Acknowledgment or _Command) 
  '' [type of E, A or C] [parameters] _CarriageReturn
  ''
  '' Wireless boards:
  '' _StartOfHeader _StartOfText (board letter*2)(origination board letter) (transmission count*3) (_Enquire, _Acknowledgment or _Command) 
  '' [type of E, A or C] [parameters] _CarriageReturn
  ''
  '' *1 Is this needed?  All wired com will go directly to one board.  Receiving board should be told the destination so it
  ''    knows if it needs to act on the communcation or just pass it on.  Probably leave in.
  '' *2 The transceiver may need to be configured to transmit to different boards.
  '' *3 Transmission count (use txCount) will increment by one each transmission so receiving board will know if data is repeated.
  ''    An ackowledgment will include the original count as a parameter.  (Use RxCountOld and RxCountNew)

  Boards A and B communications
  Since board B will only be communicating with board A the protocol will be different.
  Board B will add a small header to indicate the type of information being transmitted.
  There will be an uppercase letter to indicate the type of information followed by an colon.
  The letters supported so far are:
  A: for analytical balance data.
    After the header the weight will be transmitted ending with the character "g".
    The numbers 0-9, a decimal point "." and a leading negative may be part of the
    transmission.  The balance transmits a "+" to indicate positive weight; this character
    is ignored by board B.  There are also several space characters transmitted after the
    plus sign.  These are also ignored.
  L: for large balance data.
    After the header the weight will be transmitted ending with the character "g".
    The numbers 0-9, a decimal point "." and a leading negative may be part of the
    transmission.  The large balance does not transmit a negative sign (I don't know why).
    The "g" character indicates the end of the weight information.  The balance transmits
    additional characters after the weight which is ignored by board B.
  S: for spectrometer data.  This may need a sub header to indicate the type of spectrometer data.
    The spectrometer data will be passed on to board A with the space characters removed and a
    comma added to separate the fields.  Wave length (ending with NM), decimal number with posible
    negative sign, single letter mode indicator.  Board A will need to monitor this information to
    make sure the correct wavelength is being used and to make sure the spectrometer is in the
    correct mode.  Board A may change the mode automatically by transmitting the appropiate command
    to the spectrometer.
  B: for barcode data.
    Board B should not need to modify the data it receives from the barcode reader.  Board A
    will use the intial characters of the barcode to determine the type of container.
  E: error message.
    Possible errors include: spectrometer in the wrong mode.
    Board B should make an error sound with its piezo speaker.  
  D: not any of above, other communication.  Probably just to pass on to be displayed.
    Should everything that is displayed also be recorded?  Not for now.
    Is everthing that is displayed also spoken by the eMic module?
  X: debug input on board B.
    Just pass it along to boards C and H and record it to SC card.
  Each data packet will end with a carriage return(13).

  Wireless transmission will be acknowledged with the character R followed by the setCount variable
  sent back to originating board.

  The object pcFullDuplexSerial4FC512FlagsPlus110304a sets the acknowledge flag from within the PASM
  section of the driver (only on port 0).

  
}}  
CON'Control ASCII characters used in object.              
  {_StartOfHeader = 1
  _StartOfText = 2
  _SendToWireLess = 3          ' Used by board F to acknowledge to G a wireless Tx was send and has been received by distination
                                ' board.
  _Command = 4                 ' Followed by board destination. Use to indicate action required by other Prop such as lighting a LED
  _Enquire = 5                  ' Follow by board destination.  (A to F or higher in future)
  _Acknowledgment = 6
  _CarriageReturn = 13          ' I doubt I'll use the constant's name.  I'll just type 13.
  '' _Enquire types
  '' Also use in CheckStylusDown method as modes
  _Button = 14
  _ButtonAndKnob = 15
  _ButtonKnobAndKey = 16
  _ButtonKnobKeyAndNunchuck = 17
  '' _Command types
  _LedSingle = 18
  _LedGroup = 19
  _StopTxNunchuckToRoboni0 = 20
  _StartTxNunchuckToRoboni0 = 21  }

  #0, _Null, _StartOfHeader, _StartOfText, _SendToWireLess, _Command, _Enquire, _Acknowledgment, { 0-6

    _Enquire types, Also use in CheckStylusDown method as modes
    } _Button, _ButtonAndKnob, _ButtonKnobAndKey, _ButtonKnobKeyAndNunchuck, _LedSingle, _LedGroup, { 7-12
    } _CarriageReturn, _StopTxNunchuckToRoboni0, _StartTxNunchuckToRoboni0   {13-15} 
  
  _WirelessAck = "R"

  ' Other codes used.  These will follow one or more control characters.
  '_RtcInUseFlagF = 22
  '_RtcInUseFlagG = 23

  ' Mecanum robot commands (these will likely share numbers with other items used in communication.

  '_PlayStation2Stream = _ButtonAndKnob ' 8  No ack necessary
  '_NunchuckStream = _ButtonKnobKeyAndNunchuck ' 10  No ack necessary
  _ContollerStream = _ButtonKnobKeyAndNunchuck
  ' _ContollerStream should be followed by the type of controller
  
  ' controller enumeration
  #0, _NoController, _PlayStation2, _WiiNunchuck, _UniButtonKnobAndKey, {
    } _UniButtonKnobKeyAndNunchuck ' controller enumeration

  ' _Command types
  _DirectionVectorFloat = _ButtonKnobAndKey
  _AdjustPid = _Button
  _PidData = _ButtonAndKnob ' followed by three PID parameters coded in ASCII Hex.
  _DisplayMessage = _LedSingle ' followed by the single byte message code
  _DisplayLedText = _LedGroup ' followed by text to display
  _ChangeLedSettings = _StopTxNunchuckToRoboni0 ' followed by single byte setting code

  ' LED settings enumeration (after  _Command, _ChangeLedSettings)
  #0, _DelayUp, _DelayDown, _BrightnessUp, _BrightnessDown

  ' LED messages enumeration (after  _Command, _DisplayMessage)
  #0, _LedMessageR1, _LedMessageL1, _LedMessageR2, _LedMessageL2, { 0 - 3
    } _LedMessageP, _LedMessagePUp, _LedMessagePDown, { 4 - 6
    } _LedMessageI, _LedMessageIUp, _LedMessageIDown, { 7 - 9
    } _LedMessageD, _LedMessageDUp, _LedMessageDDown, { 10 - 12
    } _LedMessageCR, _LedMessageAllFont, _LedMessagePattern 
     
  
  ' PID enumeration
  #0, _Proportional, _Integral, _Derivative
  ' PID change enumeration (after  _Command, _AdjustPid)
  #0, _ProportionalUp, _IntegralUp, _DerivativeUp, _ProportionalDown, _IntegralDown, _DerivativeDown

CON' PlayStation 2 button enumeration
  #0, _SquarePs2, _CrossPs2, _CirclePs2, _TrianglePs2, { 0 - 3
    } _R1Ps2, _L1Ps2, _R2Ps2, _L2Ps2, { 4 - 7
    } _LeftArrowPs2, _DownArrowPs2, _RightArrowPs2, _UpArrowPs2, { 8 - 11
    } _StartPs2, _Skip13EndOfLineCharacter, _RightJoystickPs2, _LeftJoystickPs2, { 12 - 15
    } _SelectPs2 ' 16

  ' PlayStation special button meanings
  _AllMotorsStopPs2 = _CrossPs2
  '_EnableMotorsPs2 = _TrianglePs2
  '_EnterModifyPidModePs2 = _CirclePs2
  '_ExitModifyPidModePs2 = _SquarePs2
  _EnableMotorsPs2 = _StartPs2
  _EnterProgramModePs2 = _SelectPs2
  _ModifyPidModePs2 = _CirclePs2 ' must be in program mode
  _ExitProgramModePs2 = _StartPs2
  _ModifyPidProportionalPs2 = _SquarePs2 ' must be in program mode
  _ModifyPidIntegralPs2 = _TrianglePs2 ' must be in program mode
  _ModifyPidDerivativePs2 = _CirclePs2 ' must be in program mode
  _IncreaseParameterValuePs2 = _UpArrowPs2
  _DecreaseParameterValuePs2 = _DownArrowPs2
  


  _DebugTxPin = 30
  _DebugRxPin = 31
  

 
  _LargeBufferSize = 512 '128 '32
  _LargestStructureSize = 32 '_LargeBufferSize
  
  _DefaultMainDutyCycle = 2 '30
  _DefaultPerimeterDutyCycle = 10 '50
  _DefaultClockDutyCycle = 50
  _DefaultClockMode = 2         ' _ClockAmPm
  _DefaultLedFrequency = 1000
  _DefaultCheckControllerPeriod = 100 
  _DefaultMecanumRemoteLoopDelay = 4_000_000 '1/20 of a second 
  _MecanumDataSpace = 27
  _AxRawCenter = 492
  _AxJitter = 5
  _AyRawCenter = 500
  _AyJitter = 5
  _AzRawCenter = 656
  _AzBuffer = 3
  _PivotZone = 30
  _DefaultRampWheels = 50
  
  _PayloadSize = 24
  _LargestPayloadSize = 32  
  _MecanumPayloadSize = _LargestPayloadSize
  _BarcodeQueMax = 10
  _LargeBufferSizeTimesQue = _LargeBufferSize * _BarcodeQueMax
  _StackSize = 100 '64
  _TxAttempts = 32
  _DisplayTimePeriod = 300      ' five minutes



  '' Debug Mode enumeration
  #0, _DebugOff, _DebugOn



  ' transceiver enumeration
  #1, _NordicA, _NordicBaseStation, _NordicInputDevice, _NordicD, _Roboni0



  
  ' XBee settings type enumeration
  #0, _AttachedSerial, _TargetSerial

  ' Message type enumeration used in setCount
  #0, _OutgoingCount, _IncomingCount

  ' Wireless Rx Types
  #0, _UnknownRx, _NormalRx, _AcknowledgementRx


  _DefaultPayloadSize = _LargestPayloadSize
 
  _InitialAddressCharacter = $E7

  ' Memory Locations of Config Data

  _AddressSizeOffset = 3
  _StatusOffset = 7
  _PayloadOffset = 8
  _RxAddressOffset = 9

  
PUB ThrowAwayMethod